@@ -10,7 +10,7 @@ module Asciidoctor
     TAB = %(\t)
     WHITESPACE = %(#{LF}#{TAB} )
     ET = ' ' * 8
-    ESC = %(\u001b)      # troff leader marker
+    ESC = %(\u001b) # troff leader marker
     ESC_BS = %(#{ESC}\\) # escaped backslash (indicates troff formatting sequence)
     ESC_FS = %(#{ESC}.)  # escaped full stop (indicates troff macro)
 
@@ -18,8 +18,9 @@ module Asciidoctor
     LeadingPeriodRx = /^\./
     EscapedMacroRx = /^(?:#{ESC}\\c\n)?#{ESC}\.((?:URL|MTO) ".*?" ".*?" )( |[^\s]*)(.*?)(?: *#{ESC}\\c)?$/
     MockBoundaryRx = /<\/?BOUNDARY>/
-    EmDashCharRefRx = /&#8212(?:;&#8203;)?/
+    EmDashCharRefRx = /&#8212;(?:&#8203;)?/
     EllipsisCharRefRx = /&#8230;(?:&#8203;)?/
+    WrappedIndentRx = /#{CG_BLANK}*#{LF}#{CG_BLANK}*/
 
     # Converts HTML entity references back to their original form, escapes
     # special man characters and strips trailing whitespace.
@@ -28,11 +29,21 @@ module Asciidoctor
     #
     # str  - the String to convert
     # opts - an Hash of options to control processing (default: {})
-    #        * :preserve_space a Boolean that indicates whether to preserve spaces (only expanding tabs) if true
-    #          or to collapse all adjacent whitespace to a single space if false (default: true)
+    #        * :whitespace an enum that indicates how to handle whitespace; supported options are:
+    #          :preserve - preserve spaces (only expanding tabs); :normalize - normalize whitespace
+    #          (remove spaces around newlines); :collapse - collapse adjacent whitespace to a single
+    #          space (default: :collapse)
     #        * :append_newline a Boolean that indicates whether to append an endline to the result (default: false)
     def manify str, opts = {}
-      str = ((opts.fetch :preserve_space, true) ? (str.gsub TAB, ET) : (str.tr_s WHITESPACE, ' ')).
+      case opts.fetch :whitespace, :collapse
+      when :preserve
+        str = str.gsub TAB, ET
+      when :normalize
+        str = str.gsub WrappedIndentRx, LF
+      else
+        str = str.tr_s WHITESPACE, ' '
+      end
+      str = str.
         gsub(LiteralBackslashRx, '\&(rs'). # literal backslash (not a troff escape sequence)
         gsub(LeadingPeriodRx, '\\\&.'). # leading . is used in troff for macro call or other formatting; replace with \&.
         # drop orphaned \c escape lines, unescape troff macro, quote adjacent character, isolate macro line
@@ -57,15 +68,17 @@ module Asciidoctor
         gsub('&#8656;', '\(lA').  # leftwards double arrow
         gsub('&#8658;', '\(rA').  # rightwards double arrow
         gsub('&#8203;', '\:').    # zero width space
+        gsub('&amp;','&').        # literal ampersand (NOTE must take place after any other replacement that includes &)
         gsub('\'', '\(aq').       # apostrophe-quote
         gsub(MockBoundaryRx, ''). # mock boundary
         gsub(ESC_BS, '\\').       # unescape troff backslash (NOTE update if more escapes are added)
+        gsub(ESC_FS, '.').        # unescape full stop in troff commands (NOTE must take place after gsub(LeadingPeriodRx))
         rstrip                    # strip trailing space
       opts[:append_newline] ? %(#{str}#{LF}) : str
     end
 
     def skip_with_warning node, name = nil
-      warn %(asciidoctor: WARNING: converter missing for #{name || node.node_name} node in manpage backend)
+      logger.warn %(converter missing for #{name || node.node_name} node in manpage backend)
       nil
     end
 
@@ -76,19 +89,21 @@ module Asciidoctor
       mantitle = node.attr 'mantitle'
       manvolnum = node.attr 'manvolnum', '1'
       manname = node.attr 'manname', mantitle
+      manmanual = node.attr 'manmanual'
+      mansource = node.attr 'mansource'
       docdate = (node.attr? 'reproducible') ? nil : (node.attr 'docdate')
       # NOTE the first line enables the table (tbl) preprocessor, necessary for non-Linux systems
       result = [%('\\" t
 .\\"     Title: #{mantitle}
-.\\"    Author: #{(node.attr? 'authors') ? (node.attr 'authors') : '[see the "AUTHORS" section]'}
+.\\"    Author: #{(node.attr? 'authors') ? (node.attr 'authors') : '[see the "AUTHOR(S)" section]'}
 .\\" Generator: Asciidoctor #{node.attr 'asciidoctor-version'})]
       result << %(.\\"      Date: #{docdate}) if docdate
-      result << %(.\\"    Manual: #{(manual = node.attr 'manmanual') || '\ \&'}
-.\\"    Source: #{(source = node.attr 'mansource') || '\ \&'}
+      result << %(.\\"    Manual: #{manmanual ? (manmanual.tr_s WHITESPACE, ' ') : '\ \&'}
+.\\"    Source: #{mansource ? (mansource.tr_s WHITESPACE, ' ') : '\ \&'}
 .\\"  Language: English
 .\\")
       # TODO add document-level setting to disable capitalization of manname
-      result << %(.TH "#{manify manname.upcase}" "#{manvolnum}" "#{docdate}" "#{source ? (manify source) : '\ \&'}" "#{manual ? (manify manual) : '\ \&'}")
+      result << %(.TH "#{manify manname.upcase}" "#{manvolnum}" "#{docdate}" "#{mansource ? (manify mansource) : '\ \&'}" "#{manmanual ? (manify manmanual) : '\ \&'}")
       # define portability settings
       # see http://bugs.debian.org/507673
       # see http://lists.gnu.org/archive/html/groff/2009-02/msg00013.html
@@ -104,22 +119,33 @@ module Asciidoctor
       # define URL macro for portability
       # see http://web.archive.org/web/20060102165607/http://people.debian.org/~branden/talks/wtfm/wtfm.pdf
       #
-      # Use: .URL "http://www.debian.org" "Debian" "."
+      # Usage
+      #
+      # .URL "http://www.debian.org" "Debian" "."
       #
       # * First argument: the URL
       # * Second argument: text to be hyperlinked
-      # * Third (optional) argument: text that needs to immediately trail
-      #   the hyperlink without intervening whitespace
+      # * Third (optional) argument: text that needs to immediately trail the hyperlink without intervening whitespace
       result << '.de URL
-\\\\$2 \(laURL: \\\\$1 \(ra\\\\$3
+\\fI\\\\$2\\fP <\\\\$1>\\\\$3
 ..
-.if \n[.g] .mso www.tmac'
-      result << %(.LINKSTYLE #{node.attr 'man-linkstyle', 'blue R < >'})
+.als MTO URL
+.if \n[.g] \{\
+.  mso www.tmac
+.  am URL
+.    ad l
+.  .
+.  am MTO
+.    ad l
+.  .'
+      result << %(.  LINKSTYLE #{node.attr 'man-linkstyle', 'blue R < >'})
+      result << '.\}'
 
       unless node.noheader
         if node.attr? 'manpurpose'
-          result << %(.SH "#{node.attr 'manname-title'}"
-#{manify mantitle} \\- #{manify node.attr 'manpurpose'})
+          mannames = node.attr 'mannames', [manname]
+          result << %(.SH "#{(node.attr 'manname-title', 'NAME').upcase}"
+#{mannames.map {|n| manify n }.join ', '} \\- #{manify node.attr('manpurpose'), :whitespace => :normalize})
         end
       end
 
@@ -131,17 +157,21 @@ module Asciidoctor
         result.concat(node.footnotes.map {|fn| %(#{fn.index}. #{fn.text}) })
       end
 
-      # FIXME detect single author and use appropriate heading; itemize the authors if multiple
-      if node.attr? 'authors'
-        result << %(.SH "AUTHOR(S)"
+      unless (authors = node.authors).empty?
+        if authors.size > 1
+          result << '.SH "AUTHORS"'
+          authors.each do |author|
+            result << %(.sp
+#{author.name})
+          end
+        else
+          result << %(.SH "AUTHOR"
 .sp
-\\fB#{node.attr 'authors'}\\fP
-.RS 4
-Author(s).
-.RE)
+#{authors[0].name})
+        end
       end
 
-      result * LF
+      result.join LF
     end
 
     # NOTE embedded doesn't really make sense in the manpage backend
@@ -155,17 +185,14 @@ Author(s).
 
       # QUESTION should we add an AUTHOR(S) section?
 
-      result * LF
+      result.join LF
     end
 
     def section node
-      slevel = node.level
-      # QUESTION should the check for slevel be done in section?
-      slevel = 1 if slevel == 0 && node.special
       result = []
-      if slevel > 1
+      if node.level > 1
         macro = 'SS'
-        # QUESTION why captioned title? why not for slevel == 1?
+        # QUESTION why captioned title? why not when level == 1?
         stitle = node.captioned_title
       else
         macro = 'SH'
@@ -173,30 +200,28 @@ Author(s).
       end
       result << %(.#{macro} "#{manify stitle}"
 #{node.content})
-      result * LF
+      result.join LF
     end
 
     def admonition node
       result = []
-      result << %(.if n \\{\\
-.sp
-.\\}
+      result << %(.if n .sp
 .RS 4
 .it 1 an-trap
 .nr an-no-space-flag 1
 .nr an-break-flag 1
 .br
 .ps +1
-.B #{node.caption}#{node.title? ? "\\fP #{manify node.title}" : nil}
+.B #{node.attr 'textlabel'}#{node.title? ? "\\fP: #{manify node.title}" : ''}
 .ps -1
 .br
 #{resolve_content node}
 .sp .5v
 .RE)
-      result * LF
+      result.join LF
     end
 
-    alias :audio :skip_with_warning
+    alias audio skip_with_warning
 
     def colist node
       result = []
@@ -207,39 +232,43 @@ Author(s).
 tab(:);
 r lw(\n(.lu*75u/100u).'
 
-      node.items.each_with_index do |item, index|
-        result << %(\\fB(#{index + 1})\\fP\\h'-2n':T{
-#{manify item.text}
-T})
+      num = 0
+      node.items.each do |item|
+        result << %(\\fB(#{num += 1})\\fP\\h'-2n':T{)
+        result << (manify item.text, :whitespace => :normalize)
+        result << item.content if item.blocks?
+        result << 'T}'
       end
       result << '.TE'
-      result * LF
+      result.join LF
     end
 
-    # TODO implement title for dlist
     # TODO implement horizontal (if it makes sense)
     def dlist node
       result = []
+      result << %(.sp
+.B #{manify node.title}
+.br) if node.title?
       counter = 0
       node.items.each do |terms, dd|
         counter += 1
         case node.style
         when 'qanda'
           result << %(.sp
-#{counter}. #{manify([*terms].map {|dt| dt.text }.join ' ')}
+#{counter}. #{manify [*terms].map {|dt| dt.text }.join ' '}
 .RS 4)
         else
           result << %(.sp
-#{manify([*terms].map {|dt| dt.text }.join ', ')}
+#{manify [*terms].map {|dt| dt.text }.join(', '), :whitespace => :normalize}
 .RS 4)
         end
         if dd
-          result << (manify dd.text) if dd.text?
+          result << (manify dd.text, :whitespace => :normalize) if dd.text?
           result << dd.content if dd.blocks?
         end
         result << '.RE'
       end
-      result * LF
+      result.join LF
     end
 
     def example node
@@ -250,14 +279,14 @@ T})
       result << %(.RS 4
 #{resolve_content node}
 .RE)
-      result * LF
+      result.join LF
     end
 
     def floating_title node
       %(.SS "#{manify node.title}")
     end
 
-    alias :image :skip_with_warning
+    alias image skip_with_warning
 
     def listing node
       result = []
@@ -265,16 +294,12 @@ T})
 .B #{manify node.captioned_title}
 .br) if node.title?
       result << %(.sp
-.if n \\{\\
-.RS 4
-.\\}
+.if n .RS 4
 .nf
-#{manify node.content}
+#{manify node.content, :whitespace => :preserve}
 .fi
-.if n \\{\\
-.RE
-.\\})
-      result * LF
+.if n .RE)
+      result.join LF
     end
 
     def literal node
@@ -283,16 +308,12 @@ T})
 .B #{manify node.title}
 .br) if node.title?
       result << %(.sp
-.if n \\{\\
-.RS 4
-.\\}
+.if n .RS 4
 .nf
-#{manify node.content}
+#{manify node.content, :whitespace => :preserve}
 .fi
-.if n \\{\\
-.RE
-.\\})
-      result * LF
+.if n .RE)
+      result.join LF
     end
 
     def olist node
@@ -308,14 +329,14 @@ T})
 \\h'-04' #{idx + 1}.\\h'+01'\\c
 .\\}
 .el \\{\\
-.sp -1
-.IP " #{idx + 1}." 4.2
+.  sp -1
+.  IP " #{idx + 1}." 4.2
 .\\}
-#{manify item.text})
+#{manify item.text, :whitespace => :normalize})
         result << item.content if item.blocks?
         result << '.RE'
       end
-      result * LF
+      result.join LF
     end
 
     def open node
@@ -328,57 +349,55 @@ T})
     end
 
     # TODO use Page Control https://www.gnu.org/software/groff/manual/html_node/Page-Control.html#Page-Control
-    alias :page_break :skip
+    alias page_break skip
 
     def paragraph node
       if node.title?
         %(.sp
 .B #{manify node.title}
 .br
-#{manify node.content})
+#{manify node.content, :whitespace => :normalize})
       else
         %(.sp
-#{manify node.content})
+#{manify node.content, :whitespace => :normalize})
       end
     end
 
-    alias :preamble :content
+    alias preamble content
 
     def quote node
       result = []
       if node.title?
         result << %(.sp
-.in +.3i
+.RS 3
 .B #{manify node.title}
 .br
-.in)
+.RE)
       end
       attribution_line = (node.attr? 'citetitle') ? %(#{node.attr 'citetitle'} ) : nil
       attribution_line = (node.attr? 'attribution') ? %[#{attribution_line}\\(em #{node.attr 'attribution'}] : nil
-      result << %(.in +.3i
-.ll -.3i
-.nf
+      result << %(.RS 3
+.ll -.6i
 #{resolve_content node}
-.fi
 .br
-.in
+.RE
 .ll)
       if attribution_line
-        result << %(.in +.5i
-.ll -.5i
+        result << %(.RS 5
+.ll -.10i
 #{attribution_line}
-.in
+.RE
 .ll)
       end
-      result * LF
+      result.join LF
     end
 
-    alias :sidebar :skip_with_warning
+    alias sidebar skip_with_warning
 
     def stem node
       title_element = node.title? ? %(.sp
 .B #{manify node.title}
-.br) : nil
+.br) : ''
       open, close = BLOCK_MATH_DELIMITERS[node.style.to_sym]
 
       unless ((equation = node.content).start_with? open) && (equation.end_with? close)
@@ -402,15 +421,16 @@ T})
 .nr an-no-space-flag 1
 .nr an-break-flag 1
 .br
-.B #{manify node.captioned_title})
+.B #{manify node.captioned_title}
+)
       end
       result << '.TS
 allbox tab(:);'
       row_header = []
       row_text = []
       row_index = 0
-      [:head, :body, :foot].each do |tsec|
-        node.rows[tsec].each do |row|
+      node.rows.by_section.each do |tsec, rows|
+        rows.each do |row|
           row_header[row_index] ||= []
           row_text[row_index] ||= []
           # result << LF
@@ -430,19 +450,17 @@ allbox tab(:);'
               row_text[row_index] << %(T{#{LF}.sp#{LF}T}:)
             end
             row_text[row_index] << %(T{#{LF}.sp#{LF})
-            cell_halign = (cell.attr 'halign', 'left')[0..0]
+            cell_halign = (cell.attr 'halign', 'left').chr
             if tsec == :head
-              if row_header[row_index].empty? ||
-                 row_header[row_index][cell_index].empty?
+              if row_header[row_index].empty? || row_header[row_index][cell_index].empty?
                 row_header[row_index][cell_index] << %(#{cell_halign}tB)
               else
                 row_header[row_index][cell_index + 1] ||= []
                 row_header[row_index][cell_index + 1] << %(#{cell_halign}tB)
               end
-              row_text[row_index] << %(#{cell.text}#{LF})
+              row_text[row_index] << %(#{manify cell.text, :whitespace => :normalize}#{LF})
             elsif tsec == :body
-              if row_header[row_index].empty? ||
-                 row_header[row_index][cell_index].empty?
+              if row_header[row_index].empty? || row_header[row_index][cell_index].empty?
                 row_header[row_index][cell_index] << %(#{cell_halign}t)
               else
                 row_header[row_index][cell_index + 1] ||= []
@@ -451,26 +469,26 @@ allbox tab(:);'
               case cell.style
               when :asciidoc
                 cell_content = cell.content
-              when :verse, :literal
-                cell_content = cell.text
+              when :literal
+                cell_content = %(.nf#{LF}#{manify cell.text, :whitespace => :preserve}#{LF}.fi)
+              when :verse
+                cell_content = %(.nf#{LF}#{manify cell.text, :whitespace => :preserve}#{LF}.fi)
               else
-                cell_content = cell.content.join
+                cell_content = manify cell.content.join, :whitespace => :normalize
               end
               row_text[row_index] << %(#{cell_content}#{LF})
             elsif tsec == :foot
-              if row_header[row_index].empty? ||
-                 row_header[row_index][cell_index].empty?
+              if row_header[row_index].empty? || row_header[row_index][cell_index].empty?
                 row_header[row_index][cell_index] << %(#{cell_halign}tB)
               else
                 row_header[row_index][cell_index + 1] ||= []
                 row_header[row_index][cell_index + 1] << %(#{cell_halign}tB)
               end
-              row_text[row_index] << %(#{cell.text}#{LF})
+              row_text[row_index] << %(#{manify cell.text, :whitespace => :normalize}#{LF})
             end
             if cell.colspan && cell.colspan > 1
               (cell.colspan - 1).times do |i|
-                if row_header[row_index].empty? ||
-                   row_header[row_index][cell_index].empty?
+                if row_header[row_index].empty? || row_header[row_index][cell_index].empty?
                   row_header[row_index][cell_index + i] << 'st'
                 else
                   row_header[row_index][cell_index + 1 + i] ||= []
@@ -481,8 +499,7 @@ allbox tab(:);'
             if cell.rowspan && cell.rowspan > 1
               (cell.rowspan - 1).times do |i|
                 row_header[row_index + 1 + i] ||= []
-                if row_header[row_index + 1 + i].empty? ||
-                   row_header[row_index + 1 + i][cell_index].empty?
+                if row_header[row_index + 1 + i].empty? || row_header[row_index + 1 + i][cell_index].empty?
                   row_header[row_index + 1 + i][cell_index] ||= []
                   row_header[row_index + 1 + i][cell_index] << '^t'
                 else
@@ -498,7 +515,7 @@ allbox tab(:);'
             end
           end
           row_index += 1
-        end
+        end unless rows.empty?
       end
 
       #row_header.each do |row|
@@ -510,7 +527,7 @@ allbox tab(:);'
       #end
       # FIXME temporary fix to get basic table to display
       result << LF
-      result << row_header.first.map {|r| 'lt'}.join(' ')
+      result << ('lt ' * row_header[0].size).chop
 
       result << %(.#{LF})
       row_text.each do |row|
@@ -526,7 +543,7 @@ allbox tab(:);'
 \l\'\n(.lu*25u/100u\(ap\''
     end
 
-    alias :toc :skip
+    alias toc skip
 
     def ulist node
       result = []
@@ -540,14 +557,14 @@ allbox tab(:);'
 \\h'-04'\\(bu\\h'+03'\\c
 .\\}
 .el \\{\\
-.sp -1
-.IP \\(bu 2.3
+.  sp -1
+.  IP \\(bu 2.3
 .\\}
-#{manify item.text}]
+#{manify item.text, :whitespace => :normalize}]
         result << item.content if item.blocks?
         result << '.RE'
       }
-      result * LF
+      result.join LF
     end
 
     # FIXME git uses [verse] for the synopsis; detect this special case
@@ -562,7 +579,7 @@ allbox tab(:);'
       attribution_line = (node.attr? 'attribution') ? %[#{attribution_line}\\(em #{node.attr 'attribution'}] : nil
       result << %(.sp
 .nf
-#{manify node.content}
+#{manify node.content, :whitespace => :preserve}
 .fi
 .br)
       if attribution_line
@@ -572,46 +589,51 @@ allbox tab(:);'
 .in
 .ll)
       end
-      result * LF
+      result.join LF
     end
 
     def video node
-      start_param = (node.attr? 'start', nil, false) ? %(&start=#{node.attr 'start'}) : nil
-      end_param = (node.attr? 'end', nil, false) ? %(&end=#{node.attr 'end'}) : nil
-      %(.sp
-#{manify node.captioned_title} (video) <#{node.media_uri(node.attr 'target')}#{start_param}#{end_param}>)
+      start_param = (node.attr? 'start', nil, false) ? %(&start=#{node.attr 'start'}) : ''
+      end_param = (node.attr? 'end', nil, false) ? %(&end=#{node.attr 'end'}) : ''
+      result = []
+      result << %(.sp
+.B #{manify node.title}
+.br) if node.title?
+      result << %(<#{node.media_uri(node.attr 'target')}#{start_param}#{end_param}> (video))
+      result.join LF
     end
 
     def inline_anchor node
       target = node.target
       case node.type
       when :link
-        if (text = node.text) == target
-          text = nil
-        else
-          text = text.gsub '"', %[#{ESC_BS}(dq]
-        end
         if target.start_with? 'mailto:'
           macro = 'MTO'
-          target = target[7..-1].sub '@', %[#{ESC_BS}(at]
+          target = target.slice 7, target.length
         else
           macro = 'URL'
         end
+        if (text = node.text) == target
+          text = ''
+        else
+          text = text.gsub '"', %[#{ESC_BS}(dq]
+        end
+        target = target.sub '@', %[#{ESC_BS}(at] if macro == 'MTO'
         %(#{ESC_BS}c#{LF}#{ESC_FS}#{macro} "#{target}" "#{text}" )
       when :xref
         refid = (node.attr 'refid') || target
-        node.text || (node.document.references[:ids][refid] || %([#{refid}]))
+        node.text || (node.document.catalog[:ids][refid] || %([#{refid}]))
       when :ref, :bibref
-        # These are anchor points, which shouldn't be visual
+        # These are anchor points, which shouldn't be visible
         ''
       else
-        warn %(asciidoctor: WARNING: unknown anchor type: #{node.type.inspect})
+        logger.warn %(unknown anchor type: #{node.type.inspect})
+        nil
       end
     end
 
     def inline_break node
-      %(#{node.text}
-.br)
+      %(#{node.text}#{LF}#{ESC_FS}br)
     end
 
     def inline_button node
@@ -632,9 +654,7 @@ allbox tab(:);'
     end
 
     def inline_image node
-      # NOTE alt should always be set
-      alt_text = (node.attr? 'alt') ? (node.attr 'alt') : node.target
-      (node.attr? 'link') ? %([#{alt_text}] <#{node.attr 'link'}>) : %([#{alt_text}])
+      (node.attr? 'link') ? %([#{node.alt}] <#{node.attr 'link'}>) : %([#{node.alt}])
     end
 
     def inline_indexterm node
@@ -670,7 +690,7 @@ allbox tab(:);'
       when :strong
         %(#{ESC_BS}fB<BOUNDARY>#{node.text}</BOUNDARY>#{ESC_BS}fP)
       when :monospaced
-        %(#{ESC_BS}f[CR]<BOUNDARY>#{node.text}</BOUNDARY>#{ESC_BS}fP)
+        %[#{ESC_BS}f(CR<BOUNDARY>#{node.text}</BOUNDARY>#{ESC_BS}fP]
       when :single
         %[#{ESC_BS}(oq<BOUNDARY>#{node.text}</BOUNDARY>#{ESC_BS}(cq]
       when :double
@@ -681,7 +701,18 @@ allbox tab(:);'
     end
 
     def resolve_content node
-      node.content_model == :compound ? node.content : %(.sp#{LF}#{manify node.content})
+      node.content_model == :compound ? node.content : %(.sp#{LF}#{manify node.content, :whitespace => :normalize})
+    end
+
+    def write_alternate_pages mannames, manvolnum, target
+      if mannames && mannames.size > 1
+        mannames.shift
+        manvolext = %(.#{manvolnum})
+        dir, basename = ::File.split target
+        mannames.each do |manname|
+          ::IO.write ::File.join(dir, %(#{manname}#{manvolext})), %(.so #{basename})
+        end
+      end
     end
   end
 end
