@@ -783,7 +783,7 @@ include::#{url}[]
         EOS
         expect = /\{"name": "asciidoctor"\}/
         output = using_test_webserver do
-          render_embedded_string input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
+          convert_string_to_embedded input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
         end
 
         refute_nil output
@@ -797,7 +797,7 @@ include::fixtures/outer-include.adoc[]
 ....
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = 'first line of outer
 
 first line of middle
@@ -820,7 +820,7 @@ include::#{url}[]
 ....
         EOS
         output = using_test_webserver do
-          render_embedded_string input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
+          convert_string_to_embedded input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
         end
 
         expected = 'first line of outer
@@ -848,7 +848,7 @@ include::#{include_url}[]
         begin
           using_memory_logger do |logger|
             result = using_test_webserver do
-              render_embedded_string input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
+              convert_string_to_embedded input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
             end
             assert_includes result, %(Unresolved directive in #{include_url} - include::#{nested_include_url}[])
             assert_message logger, :ERROR, %(#{include_url}: line 1: include uri not readable: http://#{resolve_localhost}:9876/fixtures/#{nested_include_url}), Hash
@@ -867,7 +867,7 @@ include::#{url}[tag=init,indent=0]
 ----
         EOS
         output = using_test_webserver do
-          render_embedded_string input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
+          convert_string_to_embedded input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
         end
 
         expected = '<code class="language-ruby" data-lang="ruby">def initialize breed
@@ -887,7 +887,7 @@ include::#{url}[]
         begin
           using_memory_logger do |logger|
             output = using_test_webserver do
-              render_embedded_string input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
+              convert_string_to_embedded input, :safe => :safe, :attributes => {'allow-uri-read' => ''}
             end
             refute_nil output
             assert_match(/Unresolved directive/, output)
@@ -903,7 +903,7 @@ include::#{url}[]
 include::fixtures/include-file.asciidoc[lines=1;3..4;6..-1]
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         assert_match(/first line/, output)
         refute_match(/second line/, output)
         assert_match(/third line/, output)
@@ -920,7 +920,7 @@ include::fixtures/include-file.asciidoc[lines=1;3..4;6..-1]
 include::fixtures/include-file.asciidoc[lines="1, 3..4 , 6 .. -1"]
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         assert_match(/first line/, output)
         refute_match(/second line/, output)
         assert_match(/third line/, output)
@@ -932,6 +932,23 @@ include::fixtures/include-file.asciidoc[lines="1, 3..4 , 6 .. -1"]
         assert_match(/last line of included content/, output)
       end
 
+      test 'include directive supports implicit endless range' do
+        input = <<-EOS
+include::fixtures/include-file.asciidoc[lines=6..]
+        EOS
+
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
+        refute_match(/first line/, output)
+        refute_match(/second line/, output)
+        refute_match(/third line/, output)
+        refute_match(/fourth line/, output)
+        refute_match(/fifth line/, output)
+        assert_match(/sixth line/, output)
+        assert_match(/seventh line/, output)
+        assert_match(/eighth line/, output)
+        assert_match(/last line of included content/, output)
+      end
+
       test 'include directive ignores empty lines attribute' do
         input = <<-EOS
 ++++
@@ -939,7 +956,7 @@ include::fixtures/include-file.asciidoc[lines=]
 ++++
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         assert_includes output, 'first line of included content'
         assert_includes output, 'last line of included content'
       end
@@ -949,7 +966,7 @@ include::fixtures/include-file.asciidoc[lines=]
 include::fixtures/include-file.asciidoc[tag=snippetA]
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         assert_match(/snippetA content/, output)
         refute_match(/snippetB content/, output)
         refute_match(/non-tagged content/, output)
@@ -961,7 +978,7 @@ include::fixtures/include-file.asciidoc[tag=snippetA]
 include::fixtures/include-file.asciidoc[tags=snippetA;snippetB]
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         assert_match(/snippetA content/, output)
         assert_match(/snippetB content/, output)
         refute_match(/non-tagged content/, output)
@@ -995,7 +1012,7 @@ include::fixtures/#{filename}[tag=snippet,indent=0]
           input = <<-EOS
 include::#{tmp_include_path}[tag=include-me]
           EOS
-          output = render_embedded_string input, :safe => :safe, :base_dir => tmp_include_dir
+          output = convert_string_to_embedded input, :safe => :safe, :base_dir => tmp_include_dir
           assert_includes output, 'included line'
           refute_includes output, 'do not include'
         ensure
@@ -1003,6 +1020,26 @@ include::#{tmp_include_path}[tag=include-me]
         end 
       end
 
+      test 'include directive finds closing tag on last line of file without a trailing newline' do
+        begin
+          tmp_include = Tempfile.new %w(include- .adoc)
+          tmp_include_dir, tmp_include_path = File.split tmp_include.path
+          tmp_include.write %(line not included\ntag::include-me[]\nline included\nend::include-me[])
+          tmp_include.close
+          input = <<-EOS
+include::#{tmp_include_path}[tag=include-me]
+          EOS
+          using_memory_logger do |logger|
+            output = convert_string_to_embedded input, :safe => :safe, :base_dir => tmp_include_dir
+            assert_empty logger.messages
+            assert_includes output, 'line included'
+            refute_includes output, 'line not included'
+          end
+        ensure
+          tmp_include.close!
+        end 
+      end
+
       test 'include directive does not select lines with tag directives within selected tag region' do
         input = <<-EOS
 ++++
@@ -1010,7 +1047,7 @@ include::fixtures/include-file.asciidoc[tags=snippet]
 ++++
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expect = %(snippetA content
 
 non-tagged content
@@ -1026,7 +1063,7 @@ include::fixtures/tagged-class-enclosed.rb[tags=all;!bark]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = %(class Dog
   def initialize breed
     @breed = breed
@@ -1042,7 +1079,7 @@ include::fixtures/tagged-class.rb[tags=**]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = %(class Dog
   def initialize breed
     @breed = breed
@@ -1066,7 +1103,7 @@ include::fixtures/tagged-class.rb[tags=**;!bark]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = %(class Dog
   def initialize breed
     @breed = breed
@@ -1082,7 +1119,7 @@ include::fixtures/tagged-class-enclosed.rb[tags=*]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = %(class Dog
   def initialize breed
     @breed = breed
@@ -1106,7 +1143,7 @@ include::fixtures/tagged-class-enclosed.rb[tags=*;!init]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = %(class Dog
 
   def bark
@@ -1127,7 +1164,7 @@ include::fixtures/tagged-class.rb[tags=!*]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = %(class Dog
 end)
         assert_includes output, expected
@@ -1141,7 +1178,7 @@ include::fixtures/tagged-class.rb[tags=bark;!bark-other]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         expected = %(def bark
   if @breed == 'beagle'
     'woof woof woof woof woof'
@@ -1156,7 +1193,7 @@ include::fixtures/include-file.asciidoc[tag=no-such-tag]
         EOS
 
         using_memory_logger do |logger|
-          render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+          convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
           assert_message logger, :WARN, %(~<stdin>: line 1: tag 'no-such-tag' not found in include file), Hash
         end
       end
@@ -1169,7 +1206,7 @@ include::fixtures/include-file.asciidoc[tags=no-such-tag-b;no-such-tag-a]
         EOS
 
         using_memory_logger do |logger|
-          render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+          convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
           # NOTE Ruby 1.8 swaps the order of the list for some silly reason
           expected_tags = ::RUBY_MIN_VERSION_1_9 ? 'no-such-tag-b, no-such-tag-a' : 'no-such-tag-a, no-such-tag-b'
           assert_message logger, :WARN, %(~<stdin>: line 2: tags '#{expected_tags}' not found in include file), Hash
@@ -1184,7 +1221,7 @@ include::fixtures/unclosed-tag.adoc[tag=a]
         EOS
 
         using_memory_logger do |logger|
-          result = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+          result = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
           assert_equal 'a', result
           assert_message logger, :WARN, %(~<stdin>: line 2: detected unclosed tag 'a' starting at line 2 of include file), Hash
           refute_nil logger.messages[0][:message][:include_location]
@@ -1200,7 +1237,7 @@ include::fixtures/mismatched-end-tag.adoc[tags=a;b]
 
         inc_path = File.join DIRNAME, 'fixtures/mismatched-end-tag.adoc'
         using_memory_logger do |logger|
-          result = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+          result = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
           assert_equal %(a\nb), result
           assert_message logger, :WARN, %(<stdin>: line 2: mismatched end tag (expected 'b' but found 'a') at line 5 of include file: #{inc_path}), Hash
           refute_nil logger.messages[0][:message][:include_location]
@@ -1216,7 +1253,7 @@ include::fixtures/unexpected-end-tag.adoc[tags=a]
 
         inc_path = File.join DIRNAME, 'fixtures/unexpected-end-tag.adoc'
         using_memory_logger do |logger|
-          result = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+          result = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
           assert_equal 'a', result
           assert_message logger, :WARN, %(<stdin>: line 2: unexpected end tag 'a' at line 4 of include file: #{inc_path}), Hash
           refute_nil logger.messages[0][:message][:include_location]
@@ -1231,7 +1268,7 @@ include::fixtures/include-file.xml[#{attr_name}=]
 ++++
           EOS
 
-          output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+          output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
           assert_match(/(?:tag|end)::/, output, 2)
         end
       end
@@ -1241,7 +1278,7 @@ include::fixtures/include-file.xml[#{attr_name}=]
 include::fixtures/include-file.asciidoc[lines=1, tags=snippetA;snippetB]
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         assert_match(/first line of included content/, output)
         refute_match(/snippetA content/, output)
         refute_match(/snippetB content/, output)
@@ -1255,11 +1292,24 @@ include::fixtures/basic-docinfo.xml[lines=2..3, indent=0]
 ----
         EOS
 
-        output = render_embedded_string input, :safe => :safe, :base_dir => DIRNAME
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
         result = xmlnodes_at_xpath('//pre', output, 1).text
         assert_equal "<year>2013</year>\n<holder>Acme™, Inc.</holder>", result
       end
 
+      test 'should substitute attribute references in attrlist' do
+        input = <<-EOS
+:name-of-tag: snippetA
+include::fixtures/include-file.asciidoc[tag={name-of-tag}]
+        EOS
+
+        output = convert_string_to_embedded input, :safe => :safe, :base_dir => DIRNAME
+        assert_match(/snippetA content/, output)
+        refute_match(/snippetB content/, output)
+        refute_match(/non-tagged content/, output)
+        refute_match(/included content/, output)
+      end
+
       test 'should fall back to built-in include directive behavior when not handled by include processor' do
         input = <<-EOS
 include::fixtures/include-file.asciidoc[]
